/*
 * Copyright (C) 1996 Hellmuth Michaelis.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*---------------------------------------------------------------------------*
 *
 *	disk partition editor - defs.h
 *	------------------------------
 *
 *	last edit-date: [Sun Jan 19 19:37:15 1997]
 *
 *	-hm	cleanup
 *
 *---------------------------------------------------------------------------*/

#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <stdarg.h>
#include <ctype.h>
#include <ncurses.h>

#include <sys/param.h>
#include <sys/signal.h>
#include <sys/errno.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/mount.h>
#include <sys/wait.h>

#define DKTYPENAMES
#include <sys/disklabel.h>
#include <sys/utsname.h>

#include <ufs/ffs/fs.h>

#include "pathnames.h"
#include "control.h"

#define VERS	"0"
#define REV	"4"

#define HEADER_PART "   size      offset    type        fsize  bsize  cpg   Beg-Cyl  End-Cyl  Mbyte "
#define HEADER_HELP "Use the cursor keys to move, SPACE toggles YES/NO, CONTROL-D exits config menu "

typedef struct {
				/* the partition table */
	u_long	p_size;		/* number of sectors in partition */
	u_long	p_offset;	/* starting sector */
	u_long	p_fsize;	/* filesystem basic fragment size */
	u_long	p_fstype;	/* filesystem type */
	u_long	p_bsize;	/* filesystem basic block size */
	u_long	p_mcpg;		/* UFS: FS cylinders per group */
	u_long	p_beg;		/* start */
	u_long	p_bega;		/* aligned flag */
	u_long	p_end;		/* start */
	u_long	p_enda;		/* aligned flag */
	u_long	p_mb;		/* size in megabytes */	
} menu_partition_t;

struct menu_disklabel {
	u_long	d_magic;		/* the magic number */
	int 	d_type;			/* drive type */
	int 	d_subtype;		/* controller / d_type specific */
	char	d_typename[17];
	char	d_mpackname[17];
	char	un_d_boot0[64];		/* primary bootstrap name */
	char 	un_d_boot1[64];		/* secondary bootstrap name */
	u_long	d_secsize;		/* # of bytes per sector */
	u_long	d_nsectors;		/* # of data sectors per track */
	u_long	d_ntracks;		/* # of tracks per cylinder */
	u_long	d_ncylinders;		/* # of data cylinders per unit */
	u_long	d_secpercyl;		/* # of data sectors per cylinder */
	u_long	d_secperunit;		/* # of data sectors per unit */

	u_int	d_sparespertrack;	/* # of spare sectors per track */
	u_int	d_sparespercyl;		/* # of spare sectors per cylinder */
	u_long	d_acylinders;		/* # of alt. cylinders per unit */

	u_long	d_rpm;			/* rotational speed */
	u_int	d_interleave;		/* hardware sector interleave */
	u_int	d_trackskew;		/* sector 0 skew, per track */
	u_int	d_cylskew;		/* sector 0 skew, per cylinder */
	u_long	d_headswitch;		/* head switch time, usec */
	u_long	d_trkseek;		/* track-to-track seek, usec */
	u_long	d_flags;		/* generic flags */
	u_long	d_drivedata0;		/* drive-type specific information */
	u_long	d_drivedata1;		/* drive-type specific information */
	u_long	d_drivedata2;		/* drive-type specific information */
	u_long	d_drivedata3;		/* drive-type specific information */
	u_long	d_drivedata4;		/* drive-type specific information */	
	u_long	d_spare0;		/* reserved for future use */
	u_long	d_spare1;		/* reserved for future use */
	u_long	d_spare2;		/* reserved for future use */
	u_long	d_spare3;		/* reserved for future use */
	u_long	d_spare4;		/* reserved for future use */
	u_long	d_magic2;		/* the magic number (again) */
	u_short	d_checksum;		/* xor of data incl. partitions */
	u_short	d_npartitions;		/* number of partitions in following */
	u_long	d_bbsize;		/* size of boot area at sn0, bytes */
	u_long	d_sbsize;		/* max size of fs superblock, bytes */
	menu_partition_t part_a;
	menu_partition_t part_b;
	menu_partition_t part_c;
	menu_partition_t part_d;
	menu_partition_t part_e;
	menu_partition_t part_f;
	menu_partition_t part_g;
	menu_partition_t part_h;
};

/*---------------------------------------------------------------------------*
 *	config menu, structure for one item on the menu
 *---------------------------------------------------------------------------*/

struct item {
	int row;		/* name row position */
	int col;		/* name column position */
	char *name;		/* field name */
	int length;		/* field max length */
	int type;		/* item type */

#define ITEM_ONOFF	0		/* YES / NO */
#define ITEM_STRING	1		/* a string */
#define ITEM_CHAR	2		/* a character */
#define ITEM_INT	3		/* an integer */
#define ITEM_INDIC	4		/* an indicator */
#define ITEM_FSTYPE	5		/* FS type */
#define ITEM_DRVTYPE	6		/* disk drive type */
#define ITEM_DEVICE	7		/* which device we're on */
#define ITEM_COMMAND	8		/* what to do */
#define ITEM_STATE	9		/* mounted ? */
#define ITEM_BPS	10		/* byte per sect */
#define ITEM_FSIZE	11		/* frag size */
#define ITEM_BSIZE	12		/* block size */
#define ITEM_PSZ	13		/* partition size */
#define ITEM_PBC	14		/* partition beg cyl */
#define ITEM_PEC	15		/* partition end cyl */
#define ITEM_PMB	16		/* partition mbyte */ 

#define ITEM_MASK	0x7f		/* item mask */
#define ITEM_SPEC	0x80		/* item attribute mask */

#define ITEM_NOCHANGE	0x80		/* user cannot change item */

	int *value;		/* pointer to variable [...hmmm] */
	char *help;		/* help/description string */
	struct item *next;	/* ptr to next item struct */
	struct item *prev;	/* ptr to previous item struct */	
};

#define FIRST_I	(&i_device)	/* first menu entry, upper left */

#define UFIRST_I (&i_bps)	/* upper area, first item */
#define ULAST_I (&i_tts)	/* upper area, last item */

#define LFIRST_I (&i_pa_sz)	/* lower area, first item */
#define LLAST_I (&i_ph_mb)	/* lower area, last item */

/*---------------------------------------------------------------------------*
 *	forward declaration of individual initialized items
 *---------------------------------------------------------------------------*/

struct item i_device, i_mount, i_command;
struct item i_boot1, i_boot2;
struct item i_type, i_disk, i_label, i_flags;
struct item i_bps, i_spt, i_spc;
struct item i_tpc, i_cyl, i_spu;
struct item i_rpm, i_ilv, i_tks, i_cys;
struct item i_tts, i_hsw;
struct item i_pa_cpg, i_pa_bs, i_pa_of, i_pa_ty, i_pa_fs, i_pa_sz;
struct item i_pa_end, i_pa_enda, i_pa_beg, i_pa_bega, i_pa_mb;
struct item i_pb_cpg, i_pb_bs, i_pb_of, i_pb_ty, i_pb_fs, i_pb_sz;
struct item i_pb_end, i_pb_enda, i_pb_beg, i_pb_bega, i_pb_mb;
struct item i_pc_cpg, i_pc_bs, i_pc_of, i_pc_ty, i_pc_fs, i_pc_sz;
struct item i_pc_end, i_pc_enda, i_pc_beg, i_pc_bega, i_pc_mb;
struct item i_pd_cpg, i_pd_bs, i_pd_of, i_pd_ty, i_pd_fs, i_pd_sz;
struct item i_pd_end, i_pd_enda, i_pd_beg, i_pd_bega, i_pd_mb;
struct item i_pe_cpg, i_pe_bs, i_pe_of, i_pe_ty, i_pe_fs, i_pe_sz;
struct item i_pe_end, i_pe_enda, i_pe_beg, i_pe_bega, i_pe_mb;
struct item i_pf_cpg, i_pf_bs, i_pf_of, i_pf_ty, i_pf_fs, i_pf_sz;
struct item i_pf_end, i_pf_enda, i_pf_beg, i_pf_bega, i_pf_mb;
struct item i_pg_cpg, i_pg_bs, i_pg_of, i_pg_ty, i_pg_fs, i_pg_sz;
struct item i_pg_end, i_pg_enda, i_pg_beg, i_pg_bega, i_pg_mb;
struct item i_ph_cpg, i_ph_bs, i_ph_of, i_ph_ty, i_ph_fs, i_ph_sz;
struct item i_ph_end, i_ph_enda, i_ph_beg, i_ph_bega, i_ph_mb;

/* partition display column positions */

#define COL_SZ		0
#define COL_OF		13
#define COL_TY  	23
#define COL_FS		35
#define COL_BS		42
#define COL_CPG		49
#define COL_BEG		55
#define COL_BEGA	61
#define COL_END		64
#define COL_ENDA	70
#define COL_MB		73

/* vertical screen layout */

#define LINE_HEAD	0

/* parameter positions */

#define ROW_0		1
#define ROW_05		(ROW_0 + 1)

#define LINE_SEP0	(ROW_05 + 1)

#define ROW_1		(LINE_SEP0 + 1)
#define ROW_2		(ROW_1 + 1)
#define ROW_3		(ROW_2 + 1)
#define ROW_4		(ROW_3 + 1)

#define LINE_SEP1	(ROW_4 + 1)
#define LINE_PARTHD	(LINE_SEP1 + 1)

/* partition display column positions */

#define ROW_PA		(LINE_PARTHD + 1)
#define ROW_PB		(ROW_PA+1)
#define ROW_PC		(ROW_PB+1)
#define ROW_PD		(ROW_PC+1)
#define ROW_PE		(ROW_PD+1)
#define ROW_PF		(ROW_PE+1)
#define ROW_PG		(ROW_PF+1)
#define ROW_PH		(ROW_PG+1)

#define LINE_SEP2	(ROW_PH + 1)
#define LINE_IHELP	(LINE_SEP2 + 1)

/* field lengths */

#define LEN_SZ		9
#define LEN_OF		9
#define LEN_TY  	10
#define LEN_FS		5
#define LEN_BS		5
#define LEN_CPG		5
#define LEN_BEG		6
#define LEN_BEGA	1
#define LEN_END		6
#define LEN_ENDA	1
#define LEN_MB		6

#define RAWPARTITION	'c'

#define	NUMBOOT		2

#define	BBSIZE		8192		/* size of boot area, with label */

#define	MAXMOUNTS	256		/* max no of mounted partitions */

#define DISK_SD		"sd"		/* supported disk devices */
#define DISK_WD		"wd"

#define CMD_NOTHING	1
#define CMD_EXIT	2
#define CMD_AUTO	3
#define CMD_RD_DLF	4
#define CMD_WR_DLF	5
#define CMD_RD_DTB	6
#define CMD_WR_ALL	7
#define CMD_WR_B1	8
#define CMD_WR_B2	9
#define CMD_WR_LB	10

#ifdef MAIN

char bootarea[BBSIZE];

char *mountlist[MAXMOUNTS];
int mountmax = 0;

char specname[256] = "";	/* raw device,   i.e /dev/rwd1c		*/
char device[256] = "";		/* device name,  i.e. wd0 or sd3	*/
char specboot[256] = "";	/* device class, i.e. wd or sd		*/

int mountstate = 0;		/* mount status of selected device */

struct menu_disklabel dls;

struct disklabel disklabel;
struct disklabel *lp = &disklabel;

int	installboot;	/* non-zero if we should install a boot program */
char	*bootbuf;	/* pointer to buffer with remainder of boot prog */
int	bootsize;	/* size of remaining boot program */

char	*xxboot;	/* primary boot */
char	*bootxx;	/* secondary boot */
char	boot0[MAXPATHLEN];
char	boot1[MAXPATHLEN];

char	command[80] = "";

#else

extern struct menu_disklabel dls;

extern char bootarea[];
extern char *mountlist[];
extern int mountmax;

extern char specname[];
extern char device[];
extern char specboot[];

extern struct disklabel *lp;
extern int mountstate;

extern char *xxboot;
extern char *bootxx;
extern char boot0[];
extern char boot1[];

extern int installboot;
extern char *bootbuf;
extern int bootsize;

extern char command[];
#endif

extern void label_to_int ( struct menu_disklabel *dls, struct disklabel *lp );
extern void part_to_int ( menu_partition_t *mp, struct partition *pp, struct disklabel *lp );
extern void int_to_label ( struct menu_disklabel *dls, struct disklabel *lp );
extern void int_to_part ( menu_partition_t *mp, struct partition *pp, struct disklabel *lp );
extern void curerr ( int len, char *message );
extern int curgeti ( int len, int y, int x, char *title, int hide, int defnum );
extern void curgets ( char *str, int len, int y, int x, char *title, int hide, char *defstr );
extern int get_mounted ( void );
extern int check_mounted ( char *name );
extern void new_device ( char *name );
extern void Error ( char *str );
extern void Perror ( char *str );
extern void Warning ( char *fmt, ... );
extern u_short dkcksum ( struct disklabel *lp );
extern struct disklabel * readlabel ( int f, int rflag, struct disklabel *lp );
extern int checklabel ( struct disklabel *lp );
extern int getasciilabel ( FILE *f, struct disklabel *lp );
extern void putasciilabel ( FILE *f, struct disklabel *lp );
extern void getdisktab ( char *type, char *name, struct disklabel *lp );
extern void setbootflag ( struct disklabel *lp );
extern int writelabel ( int f, char *boot, struct disklabel *lp );
extern struct disklabel * makebootarea ( char *boot, struct disklabel *dp, int f );
extern void display_item ( struct item *ip, int inv );
extern void separate ( int row );
extern void hdrline ( int row );
extern void update_display ( void );
extern void update_upperdisplay ( void );
extern void update_lowerdisplay ( void );
extern struct item * edit_item ( struct item *ip, int c );
extern int main ( int argc, char *argv[] );
extern void display ( void );
extern struct item * edit_item ( struct item *ip, int c );
extern void display_item ( struct item *ip, int inv );
extern void label_to_int ( struct menu_disklabel *dls, struct disklabel *lp );
extern void part_to_int ( menu_partition_t *mp, struct partition *pp, struct disklabel *lp );
extern void int_to_label ( struct menu_disklabel *dls, struct disklabel *lp );
extern void int_to_part ( menu_partition_t *mp, struct partition *pp, struct disklabel *lp );
extern void update_display ( void );
extern void update_upperdisplay ( void );
extern void update_lowerdisplay ( void );
extern void execute ( int cmd );
extern int menubar ( int y, int x, char *menu[], int boxflag, char *title, int item );

/*---------------------------------- EOF ------------------------------------*/
